#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gtk/gtk.h>

#include "callbacks.h"
#include "interface.h"
#include "support.h"

/*** My additions to this file ***/

#include "xwxapt.h"

/* Text view scroller */
extern GtkWidget *gbl_text_scroller;

extern char
  gbl_samples_file[MAX_FILE_NAME+1], /* Sample file name */
  gbl_image_file  [MAX_FILE_NAME+1]; /* Image file name  */


extern int 
  gbl_duration, /* Processing duration */
  gbl_sat_type, /* Satellite type num. */
  gbl_dsp_fd;   /* DSP file descriptor */

/* Tag to wake-up timeout function */
static int cancel_timer_tag;

/* File variables */
static GtkWidget
  *quit_dialog,
  *error_dialog,
  *start_stop_timer,
  *timeout_dialog,
  *popup_menu,
  *fileselection;


/*  Error_Dialog()
 *
 *  Opens an error dialog box
 */

void
Error_Dialog( void )
{
  Cleanup();
  error_dialog = create_error_dialog();
  gtk_widget_show( error_dialog );
}

/*  File_Selection()
 *
 *  Opens a file selection dialog
 */

void
File_Selection( void )
{
  fileselection = create_fileselection();
  gtk_widget_show( fileselection );
}

/*  Cancel_Timer()
 *
 *  Handles cancellation of start-stop timer
 */

gboolean
Cancel_Timer( gpointer data )
{
  Show_Message( "Cancelling Start - Stop timer.", "black" );
  ClearFlag( START_STOP_TIMER );

  /* Take action according to flags */
  if( isFlagSet(ACTION_RECORD_APT) ||
      isFlagSet(ACTION_PROCESS_DSP) )
    {
      /* Set the buffer sync function as idle callback.  */
      /* This will make Dsp_Process_Image() as the idle  */
      /* callback if it succeeds in syncing the buffer.  */
      SetFlag( ACTION_SYNC_BUFFER );
      gtk_idle_add( Synchronize_Buffer, NULL );

    } /* if( isFlagSet(ACTION_RECORD_APT) || */
  puts("TEST");
  return( FALSE );
}


/*  Sensitize_Menu_Item()
 *
 *  Handles dynamic sensitivity changes to pop-up menu items
 */

void
Sensitize_Menu_Item( gchar *item_name, gboolean flag )
{
  GtkWidget *menu_item;

  menu_item = lookup_widget( popup_menu, item_name );
  gtk_widget_set_sensitive( menu_item, flag );
}


/*  Popup_Menu()
 *
 *  Opens pop-up menu
 */

void
Popup_Menu( void )
{
  static int first_call = TRUE;

  /* Initialize on first call */
  if( first_call )
    {
      popup_menu = create_main_menu();
      first_call = FALSE;
    }

  /* If any action is running */
  if( isFlagSet(ACTION_FLAGS_ALL) )
    {
      Sensitize_Menu_Item( "satellite_type",     FALSE );
      Sensitize_Menu_Item( "operation_timer",    FALSE );
      Sensitize_Menu_Item( "start_stop_timer",   FALSE );

      if( isFlagSet(START_STOP_TIMER) )
	Sensitize_Menu_Item( "cancel_timer",     TRUE  );
      else
	Sensitize_Menu_Item( "cancel_timer",     FALSE );

      Sensitize_Menu_Item( "set_up_audio_level", FALSE );
      Sensitize_Menu_Item( "decode_from_dsp",    FALSE );
      Sensitize_Menu_Item( "record_to_file",     FALSE );
      Sensitize_Menu_Item( "decode_from_file",   FALSE );
      Sensitize_Menu_Item( "stop", TRUE );
    }
  else /* If no action is running */
    {
      if( isFlagClear(START_STOP_TIMER) )
	{
	  Sensitize_Menu_Item( "operation_timer",    TRUE  );
	  Sensitize_Menu_Item( "start_stop_timer",   TRUE  );
	  Sensitize_Menu_Item( "cancel_timer",       FALSE );
	  Sensitize_Menu_Item( "set_up_audio_level", TRUE  );
	  Sensitize_Menu_Item( "decode_from_file",   TRUE  );
	}
      else
	{
	  Sensitize_Menu_Item( "operation_timer",    FALSE );
	  Sensitize_Menu_Item( "start_stop_timer",   FALSE );
	  Sensitize_Menu_Item( "cancel_timer",       TRUE  );
	  Sensitize_Menu_Item( "set_up_audio_level", FALSE );
	  Sensitize_Menu_Item( "decode_from_file",   FALSE );
	}

      Sensitize_Menu_Item( "satellite_type",  TRUE  );
      Sensitize_Menu_Item( "decode_from_dsp", TRUE  );
      Sensitize_Menu_Item( "record_to_file",  TRUE  );
      Sensitize_Menu_Item( "stop",            FALSE );

    } /* if( isFlagSet(ACTION_FLAGS_ALL) ) */

  gtk_menu_popup( GTK_MENU(popup_menu),
		  NULL, NULL, NULL, NULL, 3, 0);
}


/*** Glade's own functions ***/

void
on_top_window_destroy                  (GtkObject       *object,
                                        gpointer         user_data)
{
  if( isFlagSet(ACTION_FLAGS_ALL) )
    {
      quit_dialog = create_quit_dialog();
      gtk_widget_show( quit_dialog );
    }
  else
    {
      Cleanup();
      gtk_main_quit();
    }
}

gboolean
on_top_window_delete_event             (GtkWidget       *widget,
                                        GdkEvent        *event,
                                        gpointer         user_data)
{
  if( isFlagSet(ACTION_FLAGS_ALL) )
    {
      quit_dialog = create_quit_dialog();
      gtk_widget_show( quit_dialog );
    }
  else
    {
      Cleanup();
      gtk_main_quit();
    }

  return( TRUE );
}


gboolean
on_viewport1_button_press_event        (GtkWidget       *widget,
                                        GdkEventButton  *event,
                                        gpointer         user_data)
{
  switch( event->button )
    {
    case 1: /*** Toggle real time image decoding ***/
      /* If not in timeout sleep */
      if( isFlagClear(START_STOP_TIMER) )
	{      
	  /* Toggle real time image decoding */
	  if( ! isFlagSet(ACTION_FLAGS_ALL) )
	    {
	      SetFlag( ACTION_SYNC_BUFFER );
	      SetFlag( ACTION_PROCESS_DSP );

	      /* Set the buffer sync function as idle callback.  */
	      /* This will make Dsp_Process_Image() as the idle  */
	      /* callback if it succeeds in syncing the buffer.  */
	      Show_Message( "Starting Direct Decoding from dsp ...", "black" );
	      gtk_idle_add( Synchronize_Buffer, NULL );

	    } /* if( isFlagClear(ACTION_PROCESS_DSP) ) */
	  else
	    /* Toggle off all actions */
	    ClearFlag( ACTION_FLAGS_ALL );

	}  /* if( isFlagClear(START_STOP_TIMER) ) */
      else /* If in timer period, only set required action */  
	{
	  if( ! isFlagSet(ACTION_FLAGS_ALL) )
	    {
	      SetFlag( ACTION_PROCESS_DSP );
	      Show_Message( "Suspended operation: "
			    "Decode from dsp.", "black" );
	    }
	}
      break;

    case 3: /* Popup main menu */
      Popup_Menu();

    } /* switch( event->button ) */

  return FALSE;
}


gboolean
on_hscale1_button_press_event          (GtkWidget       *widget,
                                        GdkEventButton  *event,
                                        gpointer         user_data)
{
  if( isFlagSet(START_STOP_TIMER) )
    return FALSE;

  /* Run audio level setup function */
  if( ! isFlagSet(ACTION_FLAGS_ALL) )
    {
      SetFlag( ACTION_SETUP_AUDIO );
      Show_Message( "Starting the Audio Level Setup function ...", "black" );
      gtk_idle_add( Setup_Audio_Level, NULL );
    } /* if( isFlagClear(ACTION_SETUP_AUDIO) ) */
  else
    ClearFlag( ACTION_SETUP_AUDIO );

  return FALSE;
}


void
on_satellite_type_activate             (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  /* Not used */
}


void
on_noaa_activate                       (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  gbl_sat_type = NOAA;
  Initialize_Top_window();
}


void
on_meteor_activate                     (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  gbl_sat_type = METEOR;
  Initialize_Top_window();
}


void
on_operation_timer_activate            (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  timeout_dialog = create_operation_timer();
  gtk_widget_show( timeout_dialog );
}


void
on_start_stop_timer_activate           (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  start_stop_timer = create_start_stop_timer();
  gtk_widget_show( start_stop_timer );
}


void
on_cancel_timer_activate               (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  Show_Message( "Cancelling Start - Stop timer.", "black" );
  ClearFlag( START_STOP_TIMER );

  /* Take action according to flags */
  if( isFlagSet(ACTION_RECORD_APT) ||
      isFlagSet(ACTION_PROCESS_DSP) )
    {
      /* Set the buffer sync function as idle callback.  */
      /* This will make Dsp_Process_Image() as the idle  */
      /* callback if it succeeds in syncing the buffer.  */
      SetFlag( ACTION_SYNC_BUFFER );
      gtk_idle_add( Synchronize_Buffer, NULL );

    } /* if( isFlagSet(ACTION_RECORD_APT) || */

  gtk_timeout_remove( cancel_timer_tag );
}


void
on_set_up_audio_level_activate         (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  SetFlag( ACTION_SETUP_AUDIO );
  Show_Message( "Starting Audio Level Setup function ...", "black" );
  gtk_idle_add( Setup_Audio_Level, NULL );
}


void
on_decode_from_dsp_activate            (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  SetFlag( ACTION_PROCESS_DSP );

  /* Open file selection dialog */
  File_Selection();

  /* Offer a default image file name */
  File_Name( gbl_image_file );
  gtk_file_selection_set_filename( GTK_FILE_SELECTION(fileselection),
				   gbl_image_file );
}


void
on_record_to_file_activate             (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  SetFlag( ACTION_RECORD_APT );

  /* Open file selection dialog */
  File_Selection();

  /* Offer a default name for the samples file */
  File_Name( gbl_samples_file );
  gtk_file_selection_set_filename( GTK_FILE_SELECTION(fileselection),
				   gbl_samples_file );
}


void
on_decode_from_file_activate           (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  SetFlag( ACTION_PROCESS_FILE );

  /* Open file selection dialog */
  File_Selection();

  /* Offer the default 'record/' directory for opening a samples file */
  gtk_file_selection_set_filename( GTK_FILE_SELECTION(fileselection),
				   "record/" );
}


void
on_normalize_images_activate           (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  if( gtk_check_menu_item_get_active(GTK_CHECK_MENU_ITEM(menuitem)) )
    SetFlag( IMAGE_NORMALIZE );
  else
    ClearFlag( IMAGE_NORMALIZE );
}


void
on_invert_images_activate              (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  if( gtk_check_menu_item_get_active(GTK_CHECK_MENU_ITEM(menuitem)) )
    SetFlag( IMAGE_ROTATE );
  else
    ClearFlag( IMAGE_ROTATE );
}


void
on_stop_activate                       (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  ClearFlag( ACTION_FLAGS_ALL );
}


void
on_quit_activate                       (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  if( isFlagSet(ACTION_FLAGS_ALL) )
    {
      quit_dialog = create_quit_dialog();
      gtk_widget_show( quit_dialog );
    }
  else
    {
      Cleanup();
      gtk_main_quit();
    }
}


void
on_fileselection_response              (GtkDialog       *dialog,
                                        gint             response_id,
                                        gpointer         user_data)
{
  char file_name[MAX_FILE_NAME+1];
  int len;

  /* Cancel action on close or cancel */
  if( (response_id == GTK_RESPONSE_DELETE_EVENT) ||
      (response_id == GTK_RESPONSE_CANCEL) )
    {
      ClearFlag( ACTION_FLAGS_ALL );
      gtk_signal_emit_by_name( GTK_OBJECT(fileselection), "destroy" );
      return;
    }

  /* Check length of file name */
  len = strlen(gtk_file_selection_get_filename
	       (GTK_FILE_SELECTION(fileselection)));

  if( len > MAX_FILE_NAME )
    {
      Show_Message( "File name to long!", "red" );
      Error_Dialog();
      return;
    }

  /* Check for blank or globbed file name */
  strcpy( file_name, gtk_file_selection_get_filename
	  (GTK_FILE_SELECTION(fileselection)) );

  if( (file_name[len-1] == '/') ||
      (file_name[len-1] == '*') )
    {
      Show_Message( "Blank or invalid file name!", "red" );
      ClearFlag( ACTION_FLAGS_ALL );
      Error_Dialog();
      return;
    }

  /* Pass on file name according to action */
  if( isFlagSet(ACTION_PROCESS_DSP) ) /* Decode from dsp */
    strcpy( gbl_image_file, file_name );
  else /* Record or decode samples file */
    strcpy( gbl_samples_file, file_name );

  /* Close menu dialog */
  gtk_signal_emit_by_name( GTK_OBJECT(fileselection), "destroy" );

  /* Take action according to flags */
  if( isFlagClear(START_STOP_TIMER) )
    {
      if( isFlagClear(ACTION_PROCESS_FILE) )
	{
	  /* Set the buffer sync function as idle callback.  */
	  /* This will make the selected action as the idle  */
	  /* callback if it succeeds in syncing the buffer.  */
	  if( isFlagSet(ACTION_PROCESS_DSP) )
	    Show_Message( "Starting Direct Decoding from dsp ...", "black" );
	  else
	    if( isFlagSet(ACTION_RECORD_APT) )
	      Show_Message( "Starting Recording of apt signal to file ...",
			    "black" );

	  SetFlag( ACTION_SYNC_BUFFER );
	  gtk_idle_add( Synchronize_Buffer, NULL );

	} /* isFlagClear(ACTION_PROCESS_FILE) */
      else
	gtk_idle_add( File_Process_Image, NULL );

    }  /* if( isFlagClear(START_STOP_TIMER) ) */
  else /* If in timeout sleep, only show next action */
    {
      if( isFlagSet(ACTION_PROCESS_DSP) )
	Show_Message( "Suspended operation: "
		      "Decode from dsp.", "black" );
      else
	if( isFlagSet(ACTION_RECORD_APT) )
	  Show_Message( "Suspended operation: "
			"Record to file.", "black" );

    }  /* if( isFlagClear(START_STOP_TIMER) ) */
}


void
on_timeout_okbutton_clicked            (GtkButton       *button,
                                        gpointer         user_data)
{
  GtkWidget *spinbutton;

  char message[350];

  /* Set processing duration in sec */
  spinbutton = lookup_widget( timeout_dialog, "spinbutton" );
  gbl_duration = 60 * gtk_spin_button_get_value_as_int
    ( GTK_SPIN_BUTTON(spinbutton) );

  /* Warn if duration is too long */
  if( gbl_duration > 720 )
    {
      sprintf( message,
	       "Specified duration (%d sec) for this action seems excessive!",
	       gbl_duration );
      Show_Message( message, "red" );
      Error_Dialog();
    }

  gtk_signal_emit_by_name( GTK_OBJECT(timeout_dialog), "destroy" );

  Initialize_Top_window();
}


void
on_error_ok_button_clicked             (GtkButton       *button,
                                        gpointer         user_data)
{
  gtk_signal_emit_by_name( GTK_OBJECT(error_dialog), "destroy" );
}


void
on_error_quit_button_clicked           (GtkButton       *button,
                                        gpointer         user_data)
{
  Cleanup();
  gtk_main_quit();
}


void
on_hrs_entry_changed                   (GtkEditable     *editable,
                                        gpointer         user_data)
{
  gchar buff[3];

  int
    idx,
    len;

  /* Get user entry */
  strcpy( buff, gtk_entry_get_text(GTK_ENTRY(editable)) );
  len = strlen( buff );

  /* Reject non-numeric entries */
  for( idx = 0; idx < len; idx++ )
    if( (buff[idx] < '0') || (buff[idx] > '9') )
      {
	Show_Message( "Non-numeric entry!", "red" );
	Error_Dialog();
	return;
      }

  /* Reject invalid entries */
  idx = atoi( gtk_entry_get_text(GTK_ENTRY(editable)) );
  if( (idx < 0) || (idx > 23) )
    {
      Show_Message( "Value out of range!", "red" );
      Error_Dialog();
      return;
    }
}


void
on_min_entry_changed                   (GtkEditable     *editable,
                                        gpointer         user_data)
{
  gchar buff[3];

  int
    idx,
    len;

  /* Get user entry */
  strcpy( buff, gtk_entry_get_text(GTK_ENTRY(editable)) );
  len = strlen( buff );

  /* Reject non-numeric entries */
  for( idx = 0; idx < len; idx++ )
    if( (buff[idx] < '0') || (buff[idx] > '9') )
      {
	Show_Message( "Non-numeric entry!", "red" );
	Error_Dialog();
	return;
      }

  /* Reject invalid entries */
  idx = atoi( gtk_entry_get_text(GTK_ENTRY(editable)) );
  if( (idx < 0) || (idx > 59) )
    {
      Show_Message( "Value out of range!", "red" );
      Error_Dialog();
      return;
    }
}


void
on_start_stop_cancel_clicked           (GtkButton       *button,
                                        gpointer         user_data)
{
  gtk_signal_emit_by_name( GTK_OBJECT(start_stop_timer), "destroy" );
}


void
on_start_stop_ok_clicked               (GtkButton       *button,
                                        gpointer         user_data)
{
  int
    start_hrs,  /* Start time hours   */
    start_min,  /* Start time minutes */
    stop_hrs,   /* Stop time hours    */
    stop_min,   /* Stop time minutes  */
    sleep_sec,  /* Sleep time in sec  */
    stop_sec,   /* Stop time in sec since 00:00 hrs  */
    start_sec,  /* Start time in sec since 00:00 hrs */
    time_sec;   /* Time now in sec since 00:00 hrs   */

  /* Message string buffer */
  char message[350];

  /* Used to read real time */
  struct tm time_now;
  time_t t;


  /* Extract hours and minutes of start time */
  start_hrs = atoi( gtk_entry_get_text
    (GTK_ENTRY(lookup_widget(start_stop_timer, "entry1"))) );
  start_min = atoi( gtk_entry_get_text
    ( GTK_ENTRY(lookup_widget(start_stop_timer, "entry2"))) );

  /* Time now */
  t = time( &t ); 
  time_now = *gmtime( &t );
  time_sec = time_now.tm_hour*3600 +
             time_now.tm_min*60 +
             time_now.tm_sec;

  start_sec = start_hrs*3600 + start_min*60;
  sleep_sec = start_sec - time_sec;
  if( sleep_sec < 0 )
    sleep_sec += 86400; /* Next day */

  /* Extract hours and minutes of stop time */
  stop_hrs = atoi( gtk_entry_get_text
		   ( GTK_ENTRY(lookup_widget(start_stop_timer, "entry3"))) );
  stop_min = atoi( gtk_entry_get_text
		   ( GTK_ENTRY(lookup_widget(start_stop_timer, "entry4"))) );

  stop_sec  = stop_hrs*3600 + stop_min*60;
  stop_sec -= time_sec;
  if( stop_sec < 0 )
    stop_sec += 86400; /* Next day */

  /* Difference between start-stop times in sec */
  gbl_duration = stop_sec - sleep_sec;

  /* Data sanity check */
  if( gbl_duration <= 0 )
    {
      Show_Message( "Stop time seems to be ahead of start time!", "red" );
      Error_Dialog();
      return;
    }

  if( gbl_duration > 720 )
    {
      sprintf( message,
	       "Specified duration (%d sec) for this action seems excessive!",
	       gbl_duration );
      Show_Message( message, "red" );
      Error_Dialog();
      return;
    }

  gtk_signal_emit_by_name( GTK_OBJECT(start_stop_timer), "destroy" );

  /* Notify sleeping */
  sprintf( message, "Suspended till %02d:%02d. Operation timer set to %d sec.",
	   start_hrs, start_min, gbl_duration );
  Show_Message( message, "black" );

  /* Set sleep flag and wakeup timeout */
  SetFlag( START_STOP_TIMER );
  cancel_timer_tag = gtk_timeout_add
    ( sleep_sec*1000, Cancel_Timer, NULL );

  Initialize_Top_window();
}


void
on_quit_cancel_button_clicked          (GtkButton       *button,
                                        gpointer         user_data)
{
  gtk_signal_emit_by_name( GTK_OBJECT(quit_dialog), "destroy" );
}


void
on_quit_button_clicked                 (GtkButton       *button,
                                        gpointer         user_data)
{
  Cleanup();
  gtk_main_quit();
}

